@@ -23,7 +23,6 @@ import os.path
 import platform
 import configparser
 import webbrowser
-import stat
 import threading
 import time
 
@@ -39,7 +38,7 @@ if platform.system() == "Windows":
     addition = ".exe"
 else:
     addition = ""
-os.chmod(os.path.join(app_location, "bin", "iqtree" + addition), stat.S_IXOTH)
+os.chmod(os.path.join(app_location, "bin", "iqtree" + addition), 0o755)
 config = configparser.ConfigParser()
 config_path = os.path.join(root_folder, "config.ini")
 
@@ -82,19 +81,20 @@ def get_options():
     return response
 
 
-@app.route("/download/<id>", methods=["GET"])
-def download(id):
+@app.route("/download/<tree_id>", methods=["GET"])
+def download(tree_id):
     path = os.path.join(root_folder, "tmp", "download.nck")
     file = open(path, "w")
     conn = sqlite3.connect(root_folder + 'trees.db')
     c = conn.cursor()
-    tree_json = c.execute("SELECT json FROM trees WHERE id = ?", (id,)).fetchall()
-    tree_json = tree_json[0][0]
+    tree_db = c.execute("SELECT json, description FROM trees WHERE id = ?", (tree_id,)).fetchall()
+    tree_json = tree_db[0][0]
+    tree_description = tree_db[0][1]
     tree = Tree(tree_json).to_newick()
     file.write(tree + "\n")
     file.close()
     return send_from_directory(os.path.join(root_folder, "tmp"), "download.nck", as_attachment=True,
-                               attachment_filename=id + ".nck")
+                               attachment_filename=tree_description + ".nck")
 
 
 @app.route("/description", methods=["POST"])
@@ -256,6 +256,14 @@ def load():
     return response
 
 
+@app.route("/reset/<tree_id>", methods=["GET"])
+def reset(tree_id):
+    session["trees"] = [tree_id]
+    response = make_response("OK")
+    response.headers["Cache-Control"] = "no-store"
+    return response
+
+
 @app.route("/options", methods=["POST"])
 def options():
     config.read(config_path)
